@@ -49,22 +49,6 @@
 #include "mischelp.h"
 
 
-void
-wipememory (void *ptr, size_t len)
-{
-#if defined(HAVE_W32_SYSTEM) && defined(SecureZeroMemory)
-  SecureZeroMemory (ptr, len);
-#elif defined(HAVE_EXPLICIT_BZERO)
-  explicit_bzero (ptr, len);
-#else
-  /* Prevent compiler from optimizing away the call to memset by accessing
-     memset through volatile pointer. */
-  static void *(*volatile memset_ptr)(void *, int, size_t) = (void *)memset;
-  memset_ptr (ptr, 0, len);
-#endif
-}
-
-
 /* Check whether the files NAME1 and NAME2 are identical.  This is for
    example achieved by comparing the inode numbers of the files.  */
 int
@@ -80,30 +64,35 @@ same_file_p (const char *name1, const char *name2)
 #ifdef HAVE_W32_SYSTEM
       HANDLE file1, file2;
       BY_HANDLE_FILE_INFORMATION info1, info2;
-      wchar_t *wname;
-
-      wname = gpgrt_fname_to_wchar (name1);
-      if (wname)
-        {
-          file1 = CreateFileW (wname, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
-          xfree (wname);
-        }
-      else
-        file1 = INVALID_HANDLE_VALUE;
 
+#ifdef HAVE_W32CE_SYSTEM
+      {
+        wchar_t *wname = utf8_to_wchar (name1);
+        if (wname)
+          file1 = CreateFile (wname, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
+        else
+          file1 = INVALID_HANDLE_VALUE;
+        xfree (wname);
+      }
+#else
+      file1 = CreateFile (name1, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
+#endif
       if (file1 == INVALID_HANDLE_VALUE)
         yes = 0; /* If we can't open the file, it is not the same.  */
       else
         {
-          wname = gpgrt_fname_to_wchar (name2);
-          if (wname)
-            {
-              file2 = CreateFileW (wname, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
-              xfree (wname);
-            }
-          else
-            file2 = INVALID_HANDLE_VALUE;
-
+#ifdef HAVE_W32CE_SYSTEM
+          {
+            wchar_t *wname = utf8_to_wchar (name2);
+            if (wname)
+              file2 = CreateFile (wname, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
+            else
+              file2 = INVALID_HANDLE_VALUE;
+            xfree (wname);
+          }
+#else
+          file2 = CreateFile (name2, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
+#endif
           if (file2 == INVALID_HANDLE_VALUE)
             yes = 0; /* If we can't open the file, it is not the same.  */
           else
