@@ -2,7 +2,7 @@
  * Copyright (C) 1998, 1999, 2000, 2001, 2003,
  *               2004, 2006, 2010 Free Software Foundation, Inc.
  * Copyright (C) 2014 Werner Koch
- * Copyright (C) 2016, 2023 g10 Code GmbH
+ * Copyright (C) 2016 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -18,7 +18,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
- * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #include <config.h>
@@ -69,12 +68,12 @@ pubkey_letter( int algo )
 }
 
 /* Return a string describing the public key algorithm and the
-   keysize.  For elliptic curves the function prints the name of the
+   keysize.  For elliptic curves the functions prints the name of the
    curve because the keysize is a property of the curve.  The string
    is copied to the supplied buffer up a length of BUFSIZE-1.
    Examples for the output are:
 
-   "rsa3072"  - RSA with 3072 bit
+   "rsa2048"  - RSA with 2048 bit
    "elg1024"  - Elgamal with 1024 bit
    "ed25519"  - ECC using the curve Ed25519.
    "E_1.2.3.4"  - ECC using the unsupported curve with OID "1.2.3.4".
@@ -84,15 +83,12 @@ pubkey_letter( int algo )
    If the option --legacy-list-mode is active, the output use the
    legacy format:
 
-   "3072R" - RSA with 3072 bit
+   "2048R" - RSA with 2048 bit
    "1024g" - Elgamal with 1024 bit
    "256E"  - ECDSA using a curve with 256 bit
 
    The macro PUBKEY_STRING_SIZE may be used to allocate a buffer with
-   a suitable size.  Note that a more general version of this function
-   exists as get_keyalgo_string.  However, that has no special
-   treatment for the old and unsupported Elgamal which we here print as
-   xxxNNNN.  */
+   a suitable size.*/
 char *
 pubkey_string (PKT_public_key *pk, char *buffer, size_t bufsize)
 {
@@ -140,13 +136,12 @@ pubkey_string (PKT_public_key *pk, char *buffer, size_t bufsize)
 }
 
 
-/* Hash a public key and allow to specify the to be used format.
- * Note that if the v5 format is requested for a v4 key, a 0x04 as
- * version is hashed instead of the 0x05. */
-static void
-do_hash_public_key (gcry_md_hd_t md, PKT_public_key *pk, int use_v5)
+/* Hash a public key.  This function is useful for v4 fingerprints and
+   for v3 or v4 key signing. */
+void
+hash_public_key (gcry_md_hd_t md, PKT_public_key *pk)
 {
-  unsigned int n;
+  unsigned int n = 6;
   unsigned int nn[PUBKEY_MAX_NPKEY];
   byte *pp[PUBKEY_MAX_NPKEY];
   int i;
@@ -154,7 +149,6 @@ do_hash_public_key (gcry_md_hd_t md, PKT_public_key *pk, int use_v5)
   size_t nbytes;
   int npkey = pubkey_get_npkey (pk->pubkey_algo);
 
-  n = use_v5? 10 : 6;
   /* FIXME: We can avoid the extra malloc by calling only the first
      mpi_print here which computes the required length and calling the
      real mpi_print only at the end.  The speed advantage would only be
@@ -181,39 +175,15 @@ do_hash_public_key (gcry_md_hd_t md, PKT_public_key *pk, int use_v5)
             }
           else if (gcry_mpi_get_flag (pk->pkey[i], GCRYMPI_FLAG_OPAQUE))
             {
-              const char *p;
-              int is_sos = 0;
-
-              if (gcry_mpi_get_flag (pk->pkey[i], GCRYMPI_FLAG_USER2))
-                is_sos = 2;
+              const void *p;
 
               p = gcry_mpi_get_opaque (pk->pkey[i], &nbits);
-              pp[i] = xmalloc ((nbits+7)/8 + is_sos);
+              pp[i] = xmalloc ((nbits+7)/8);
               if (p)
-                memcpy (pp[i] + is_sos, p, (nbits+7)/8);
+                memcpy (pp[i], p, (nbits+7)/8);
               else
                 pp[i] = NULL;
-              if (is_sos)
-                {
-                  if (*p)
-                    {
-                      nbits = ((nbits + 7) / 8) * 8;
-
-                      if (nbits >= 8 && !(*p & 0x80))
-                        if (--nbits >= 7 && !(*p & 0x40))
-                          if (--nbits >= 6 && !(*p & 0x20))
-                            if (--nbits >= 5 && !(*p & 0x10))
-                              if (--nbits >= 4 && !(*p & 0x08))
-                                if (--nbits >= 3 && !(*p & 0x04))
-                                  if (--nbits >= 2 && !(*p & 0x02))
-                                    if (--nbits >= 1 && !(*p & 0x01))
-                                      --nbits;
-                    }
-
-                  pp[i][0] = (nbits >> 8);
-                  pp[i][1] = nbits;
-                }
-              nn[i] = (nbits+7)/8 + is_sos;
+              nn[i] = (nbits+7)/8;
               n += nn[i];
             }
           else
@@ -231,23 +201,12 @@ do_hash_public_key (gcry_md_hd_t md, PKT_public_key *pk, int use_v5)
         }
     }
 
-  if (use_v5)
-    {
-      gcry_md_putc ( md, 0x9a );     /* ctb */
-      gcry_md_putc ( md, n >> 24 );  /* 4 byte length header */
-      gcry_md_putc ( md, n >> 16 );
-      gcry_md_putc ( md, n >>  8 );
-      gcry_md_putc ( md, n       );
-      /* Note that the next byte may either be 4 or 5.  */
-      gcry_md_putc ( md, pk->version );
-    }
-  else
-    {
-      gcry_md_putc ( md, 0x99 );     /* ctb */
-      gcry_md_putc ( md, n >> 8 );   /* 2 byte length header */
-      gcry_md_putc ( md, n );
-      gcry_md_putc ( md, pk->version );
-    }
+  gcry_md_putc ( md, 0x99 );     /* ctb */
+  /* What does it mean if n is greater than 0xFFFF ? */
+  gcry_md_putc ( md, n >> 8 );   /* 2 byte length header */
+  gcry_md_putc ( md, n );
+  gcry_md_putc ( md, pk->version );
+
   gcry_md_putc ( md, pk->timestamp >> 24 );
   gcry_md_putc ( md, pk->timestamp >> 16 );
   gcry_md_putc ( md, pk->timestamp >>  8 );
@@ -255,15 +214,6 @@ do_hash_public_key (gcry_md_hd_t md, PKT_public_key *pk, int use_v5)
 
   gcry_md_putc ( md, pk->pubkey_algo );
 
-  if (use_v5)
-    {
-      n -= 10;
-      gcry_md_putc ( md, n >> 24 );
-      gcry_md_putc ( md, n >> 16 );
-      gcry_md_putc ( md, n >>  8 );
-      gcry_md_putc ( md, n       );
-    }
-
   if(npkey==0 && pk->pkey[0]
      && gcry_mpi_get_flag (pk->pkey[0], GCRYMPI_FLAG_OPAQUE))
     {
@@ -282,12 +232,17 @@ do_hash_public_key (gcry_md_hd_t md, PKT_public_key *pk, int use_v5)
 }
 
 
-/* Hash a public key.  This function is useful for v4 and v5
- * fingerprints and for v3 or v4 key signing. */
-void
-hash_public_key (gcry_md_hd_t md, PKT_public_key *pk)
+static gcry_md_hd_t
+do_fingerprint_md( PKT_public_key *pk )
 {
-  do_hash_public_key (md, pk, (pk->version == 5));
+  gcry_md_hd_t md;
+
+  if (gcry_md_open (&md, DIGEST_ALGO_SHA1, 0))
+    BUG ();
+  hash_public_key(md,pk);
+  gcry_md_final( md );
+
+  return md;
 }
 
 
@@ -368,12 +323,11 @@ keyid_copy (u32 *dest, const u32 *src)
 char *
 format_keyid (u32 *keyid, int format, char *buffer, int len)
 {
+  char tmp[KEYID_STR_SIZE];
   if (! buffer)
     {
-      len = KEYID_STR_SIZE;
-      buffer = xtrymalloc (len);
-      if (!buffer)
-        return NULL;
+      buffer = tmp;
+      len = sizeof (tmp);
     }
 
   if (format == KF_DEFAULT)
@@ -408,6 +362,8 @@ format_keyid (u32 *keyid, int format, char *buffer, int len)
       BUG();
     }
 
+  if (buffer == tmp)
+    return xstrdup (buffer);
   return buffer;
 }
 
@@ -516,93 +472,70 @@ keystr_from_desc(KEYDB_SEARCH_DESC *desc)
     case KEYDB_SEARCH_MODE_SHORT_KID:
       return keystr(desc->u.kid);
 
-    case KEYDB_SEARCH_MODE_FPR:
+    case KEYDB_SEARCH_MODE_FPR20:
       {
 	u32 keyid[2];
 
-        if (desc->fprlen == 32)
-          {
-            keyid[0] = buf32_to_u32 (desc->u.fpr);
-            keyid[1] = buf32_to_u32 (desc->u.fpr+4);
-          }
-        else if (desc->fprlen == 20)
-          {
-            keyid[0] = buf32_to_u32 (desc->u.fpr+12);
-            keyid[1] = buf32_to_u32 (desc->u.fpr+16);
-          }
-        else if (desc->fprlen == 16)
-          return "?v3 fpr?";
-        else /* oops */
-          return "?vx fpr?";
+	keyid[0] = buf32_to_u32 (desc->u.fpr+12);
+	keyid[1] = buf32_to_u32 (desc->u.fpr+16);
 	return keystr(keyid);
       }
 
+    case KEYDB_SEARCH_MODE_FPR16:
+      return "?v3 fpr?";
+
     default:
       BUG();
     }
 }
 
 
-/* Compute the fingerprint and keyid and store it in PK.  */
-static void
-compute_fingerprint (PKT_public_key *pk)
-{
-  const byte *dp;
-  gcry_md_hd_t md;
-  size_t len;
-
-  if (gcry_md_open (&md, pk->version == 5 ? GCRY_MD_SHA256 : GCRY_MD_SHA1, 0))
-    BUG ();
-  hash_public_key (md, pk);
-  gcry_md_final (md);
-  dp = gcry_md_read (md, 0);
-  len = gcry_md_get_algo_dlen (gcry_md_get_algo (md));
-  log_assert (len <= MAX_FINGERPRINT_LEN);
-  memcpy (pk->fpr, dp, len);
-  pk->fprlen = len;
-  if (pk->version == 5)
-    {
-      pk->keyid[0] = buf32_to_u32 (dp);
-      pk->keyid[1] = buf32_to_u32 (dp+4);
-    }
-  else
-    {
-      pk->keyid[0] = buf32_to_u32 (dp+12);
-      pk->keyid[1] = buf32_to_u32 (dp+16);
-    }
-  gcry_md_close( md);
-}
-
-
 /*
- * Get the keyid from the public key PK and store it at KEYID unless
- * this is NULL.  Returns the 32 bit short keyid.
+ * Get the keyid from the public key and put it into keyid
+ * if this is not NULL. Return the 32 low bits of the keyid.
  */
 u32
 keyid_from_pk (PKT_public_key *pk, u32 *keyid)
 {
+  u32 lowbits;
   u32 dummy_keyid[2];
 
   if (!keyid)
     keyid = dummy_keyid;
 
-  if (!pk->fprlen)
-    compute_fingerprint (pk);
-
-  keyid[0] = pk->keyid[0];
-  keyid[1] = pk->keyid[1];
-
-  if (pk->fprlen == 32)
-    return keyid[0];
+  if( pk->keyid[0] || pk->keyid[1] )
+    {
+      keyid[0] = pk->keyid[0];
+      keyid[1] = pk->keyid[1];
+      lowbits = keyid[1];
+    }
   else
-    return keyid[1];
+    {
+      const byte *dp;
+      gcry_md_hd_t md;
+
+      md = do_fingerprint_md(pk);
+      if(md)
+	{
+	  dp = gcry_md_read ( md, 0 );
+	  keyid[0] = buf32_to_u32 (dp+12);
+	  keyid[1] = buf32_to_u32 (dp+16);
+	  lowbits = keyid[1];
+	  gcry_md_close (md);
+	  pk->keyid[0] = keyid[0];
+	  pk->keyid[1] = keyid[1];
+	}
+      else
+	pk->keyid[0]=pk->keyid[1]=keyid[0]=keyid[1]=lowbits=0xFFFFFFFF;
+    }
+
+  return lowbits;
 }
 
 
 /*
- * Get the keyid from the fingerprint.  This function is simple for
- * most keys, but has to do a key lookup for old v3 keys where the
- * keyid is not part of the fingerprint.
+ * Get the keyid from the fingerprint.	This function is simple for most
+ * keys, but has to do a keylookup for old stayle keys.
  */
 u32
 keyid_from_fingerprint (ctrl_t ctrl, const byte *fprint,
@@ -613,7 +546,7 @@ keyid_from_fingerprint (ctrl_t ctrl, const byte *fprint,
   if( !keyid )
     keyid = dummy_keyid;
 
-  if (fprint_len != 20 && fprint_len != 32)
+  if (fprint_len != 20)
     {
       /* This is special as we have to lookup the key first.  */
       PKT_public_key pk;
@@ -623,8 +556,7 @@ keyid_from_fingerprint (ctrl_t ctrl, const byte *fprint,
       rc = get_pubkey_byfprint (ctrl, &pk, NULL, fprint, fprint_len);
       if( rc )
         {
-          log_printhex (fprint, fprint_len,
-                        "Oops: keyid_from_fingerprint: no pubkey; fpr:");
+          log_error("Oops: keyid_from_fingerprint: no pubkey\n");
           keyid[0] = 0;
           keyid[1] = 0;
         }
@@ -634,16 +566,8 @@ keyid_from_fingerprint (ctrl_t ctrl, const byte *fprint,
   else
     {
       const byte *dp = fprint;
-      if (fprint_len == 20)  /* v4 key */
-        {
-          keyid[0] = buf32_to_u32 (dp+12);
-          keyid[1] = buf32_to_u32 (dp+16);
-        }
-      else  /* v5 key */
-        {
-          keyid[0] = buf32_to_u32 (dp);
-          keyid[1] = buf32_to_u32 (dp+4);
-        }
+      keyid[0] = buf32_to_u32 (dp+12);
+      keyid[1] = buf32_to_u32 (dp+16);
     }
 
   return keyid[1];
@@ -658,7 +582,7 @@ keyid_from_sig (PKT_signature *sig, u32 *keyid)
       keyid[0] = sig->keyid[0];
       keyid[1] = sig->keyid[1];
     }
-  return sig->keyid[1];  /*FIXME:shortkeyid*/
+  return sig->keyid[1];
 }
 
 
@@ -716,7 +640,7 @@ mk_datestr (char *buffer, size_t bufsize, u32 timestamp)
  *    Format is: yyyy-mm-dd
  */
 const char *
-dateonlystr_from_pk (PKT_public_key *pk)
+datestr_from_pk (PKT_public_key *pk)
 {
   static char buffer[MK_DATESTR_SIZE];
 
@@ -724,36 +648,14 @@ dateonlystr_from_pk (PKT_public_key *pk)
 }
 
 
-/* Same as dateonlystr_from_pk but with a global option a full iso
- * timestamp is returned.  In this case it shares a static buffer with
- * isotimestamp(). */
 const char *
-datestr_from_pk (PKT_public_key *pk)
-{
-  if (opt.flags.full_timestrings)
-    return isotimestamp (pk->timestamp);
-  else
-    return dateonlystr_from_pk (pk);
-}
-
-
-const char *
-dateonlystr_from_sig (PKT_signature *sig )
+datestr_from_sig (PKT_signature *sig )
 {
   static char buffer[MK_DATESTR_SIZE];
 
   return mk_datestr (buffer, sizeof buffer, sig->timestamp);
 }
 
-const char *
-datestr_from_sig (PKT_signature *sig )
-{
-  if (opt.flags.full_timestrings)
-    return isotimestamp (sig->timestamp);
-  else
-    return dateonlystr_from_sig (sig);
-}
-
 
 const char *
 expirestr_from_pk (PKT_public_key *pk)
@@ -762,10 +664,6 @@ expirestr_from_pk (PKT_public_key *pk)
 
   if (!pk->expiredate)
     return _("never     ");
-
-  if (opt.flags.full_timestrings)
-    return isotimestamp (pk->expiredate);
-
   return mk_datestr (buffer, sizeof buffer, pk->expiredate);
 }
 
@@ -777,10 +675,6 @@ expirestr_from_sig (PKT_signature *sig)
 
   if (!sig->expiredate)
     return _("never     ");
-
-  if (opt.flags.full_timestrings)
-    return isotimestamp (sig->expiredate);
-
   return mk_datestr (buffer, sizeof buffer, sig->expiredate);
 }
 
@@ -792,10 +686,6 @@ revokestr_from_pk( PKT_public_key *pk )
 
   if(!pk->revoked.date)
     return _("never     ");
-
-  if (opt.flags.full_timestrings)
-    return isotimestamp (pk->revoked.date);
-
   return mk_datestr (buffer, sizeof buffer, pk->revoked.date);
 }
 
@@ -819,13 +709,6 @@ usagestr_from_pk (PKT_public_key *pk, int fill)
   if ( (use & PUBKEY_USAGE_AUTH) )
     buffer[i++] = 'A';
 
-  if ( (use & PUBKEY_USAGE_RENC) )
-    buffer[i++] = 'R';
-  if ( (use & PUBKEY_USAGE_TIME) )
-    buffer[i++] = 'T';
-  if ( (use & PUBKEY_USAGE_GROUP) )
-    buffer[i++] = 'G';
-
   while (fill && i < 4)
     buffer[i++] = ' ';
 
@@ -877,7 +760,6 @@ colon_expirestr_from_sig (PKT_signature *sig)
 }
 
 
-
 /*
  * Return a byte array with the fingerprint for the given PK/SK
  * The length of the array is returned in ret_len. Caller must free
@@ -885,83 +767,28 @@ colon_expirestr_from_sig (PKT_signature *sig)
  */
 byte *
 fingerprint_from_pk (PKT_public_key *pk, byte *array, size_t *ret_len)
-{
-  if (!pk->fprlen)
-    compute_fingerprint (pk);
-
-  if (!array)
-    array = xmalloc (pk->fprlen);
-  memcpy (array, pk->fpr, pk->fprlen);
-
-  if (ret_len)
-    *ret_len = pk->fprlen;
-  return array;
-}
-
-
-/*
- * Return a byte array with the fingerprint for the given PK/SK The
- * length of the array is returned in ret_len. Caller must free the
- * array or provide an array of length MAX_FINGERPRINT_LEN.  This
- * version creates a v5 fingerprint even vor v4 keys.
- */
-byte *
-v5_fingerprint_from_pk (PKT_public_key *pk, byte *array, size_t *ret_len)
 {
   const byte *dp;
+  size_t len;
   gcry_md_hd_t md;
 
-  if (pk->version == 5)
-    return fingerprint_from_pk (pk, array, ret_len);
-
-  if (gcry_md_open (&md, GCRY_MD_SHA256, 0))
-    BUG ();
-  do_hash_public_key (md, pk, 1);
-  gcry_md_final (md);
-  dp = gcry_md_read (md, 0);
+  md = do_fingerprint_md(pk);
+  dp = gcry_md_read( md, 0 );
+  len = gcry_md_get_algo_dlen (gcry_md_get_algo (md));
+  log_assert( len <= MAX_FINGERPRINT_LEN );
   if (!array)
-    array = xmalloc (32);
-  memcpy (array, dp, 32);
-  gcry_md_close (md);
+    array = xmalloc ( len );
+  memcpy (array, dp, len );
+  pk->keyid[0] = buf32_to_u32 (dp+12);
+  pk->keyid[1] = buf32_to_u32 (dp+16);
+  gcry_md_close( md);
 
   if (ret_len)
-    *ret_len = 32;
+    *ret_len = len;
   return array;
 }
 
 
-/*
- * Get FPR20 for the given PK/SK into ARRAY.
- *
- * FPR20 is special form of fingerprint of length 20 for the record of
- * trustdb.  For v4key, having fingerprint with SHA-1, FPR20 is the
- * same one.  For v5key, FPR20 is constructed from its fingerprint
- * with SHA-2, so that its kid of last 8-byte can be as same as
- * kid of v5key fingerprint.
- *
- */
-void
-fpr20_from_pk (PKT_public_key *pk, byte array[20])
-{
-  if (!pk->fprlen)
-    compute_fingerprint (pk);
-
-  if (!array)
-    array = xmalloc (pk->fprlen);
-
-  if (pk->fprlen == 32)         /* v5 fingerprint */
-    {
-      memcpy (array +  0, pk->fpr + 20, 4);
-      memcpy (array +  4, pk->fpr + 24, 4);
-      memcpy (array +  8, pk->fpr + 28, 4);
-      memcpy (array + 12, pk->fpr +  0, 4); /* kid[0] */
-      memcpy (array + 16, pk->fpr +  4, 4); /* kid[1] */
-    }
-  else                          /* v4 fingerprint */
-    memcpy (array, pk->fpr, 20);
-}
-
-
 /* Return an allocated buffer with the fingerprint of PK formatted as
  * a plain hexstring.  If BUFFER is NULL the result is a malloc'd
  * string.  If BUFFER is not NULL the result will be copied into this
@@ -972,47 +799,23 @@ fpr20_from_pk (PKT_public_key *pk, byte array[20])
 char *
 hexfingerprint (PKT_public_key *pk, char *buffer, size_t buflen)
 {
-  if (!pk->fprlen)
-    compute_fingerprint (pk);
+  unsigned char fpr[MAX_FINGERPRINT_LEN];
+  size_t len;
 
+  fingerprint_from_pk (pk, fpr, &len);
   if (!buffer)
     {
-      buffer = xtrymalloc (2 * pk->fprlen + 1);
+      buffer = xtrymalloc (2 * len + 1);
       if (!buffer)
         return NULL;
     }
-  else if (buflen < 2 * pk->fprlen + 1)
+  else if (buflen < 2*len+1)
     log_fatal ("%s: buffer too short (%zu)\n", __func__, buflen);
-
-  bin2hex (pk->fpr, pk->fprlen, buffer);
+  bin2hex (fpr, len, buffer);
   return buffer;
 }
 
 
-/* Same as hexfingerprint but returns a v5 fingerprint also for a v4
- * key.  */
-char *
-v5hexfingerprint (PKT_public_key *pk, char *buffer, size_t buflen)
-{
-  char fprbuf[32];
-
-  if (pk->version == 5)
-    return hexfingerprint (pk, buffer, buflen);
-
-  if (!buffer)
-    {
-      buffer = xtrymalloc (2 * 32 + 1);
-      if (!buffer)
-        return NULL;
-    }
-  else if (buflen < 2 * 32 + 1)
-    log_fatal ("%s: buffer too short (%zu)\n", __func__, buflen);
-
-  v5_fingerprint_from_pk (pk, fprbuf, NULL);
-  return bin2hex (fprbuf, 32, buffer);
-}
-
-
 /* Pretty print a hex fingerprint.  If BUFFER is NULL the result is a
    malloc'd string.  If BUFFER is not NULL the result will be copied
    into this buffer.  In the latter case BUFLEN describes the length
@@ -1036,22 +839,8 @@ format_hexfingerprint (const char *fingerprint, char *buffer, size_t buflen)
 	       /* Half way through we add a second space.  */
 	       + 1);
     }
-  else if (hexlen == 64 || hexlen == 50)  /* v5 fingerprint */
-    {
-      /* The v5 fingerprint is commonly printed truncated to 25
-       * octets.  We accept the truncated as well as the full hex
-       * version here and format it like this:
-       * 19347 BC987 24640 25F99 DF3EC 2E000 0ED98 84892 E1F7B 3EA4C
-       */
-      hexlen = 50;
-      space = 10 * 5 + 9 + 1;
-    }
   else  /* Other fingerprint versions - print as is.  */
     {
-      /* We truncated here so that we do not need to provide a buffer
-       * of a length which is in reality never used.  */
-      if (hexlen > MAX_FORMATTED_FINGERPRINT_LEN - 1)
-        hexlen = MAX_FORMATTED_FINGERPRINT_LEN - 1;
       space = hexlen + 1;
     }
 
@@ -1064,7 +853,7 @@ format_hexfingerprint (const char *fingerprint, char *buffer, size_t buflen)
     {
       for (i = 0, j = 0; i < 40; i ++)
         {
-          if (i && !(i % 4))
+          if (i && i % 4 == 0)
             buffer[j ++] = ' ';
           if (i == 40 / 2)
             buffer[j ++] = ' ';
@@ -1074,20 +863,9 @@ format_hexfingerprint (const char *fingerprint, char *buffer, size_t buflen)
       buffer[j ++] = 0;
       log_assert (j == space);
     }
-  else if (hexlen == 50)  /* v5 fingerprint */
-    {
-      for (i=j=0; i < 50; i++)
-        {
-          if (i && !(i % 5))
-            buffer[j++] = ' ';
-          buffer[j++] = fingerprint[i];
-        }
-      buffer[j++] = 0;
-      log_assert (j == space);
-    }
   else
     {
-      mem2str (buffer, fingerprint, space);
+      strcpy (buffer, fingerprint);
     }
 
   return buffer;
@@ -1096,7 +874,7 @@ format_hexfingerprint (const char *fingerprint, char *buffer, size_t buflen)
 
 
 /* Return the so called KEYGRIP which is the SHA-1 hash of the public
-   key parameters expressed as an canonical encoded S-Exp.  ARRAY must
+   key parameters expressed as an canoncial encoded S-Exp.  ARRAY must
    be 20 bytes long.  Returns 0 on success or an error code.  */
 gpg_error_t
 keygrip_from_pk (PKT_public_key *pk, unsigned char *array)
@@ -1163,19 +941,14 @@ keygrip_from_pk (PKT_public_key *pk, unsigned char *array)
 
   if (!gcry_pk_get_keygrip (s_pkey, array))
     {
-      char *hexfpr;
-
-      hexfpr = hexfingerprint (pk, NULL, 0);
-      log_info ("error computing keygrip (fpr=%s)\n", hexfpr);
-      xfree (hexfpr);
-
+      log_info ("error computing keygrip\n");
       memset (array, 0, 20);
       err = gpg_error (GPG_ERR_GENERAL);
     }
   else
     {
       if (DBG_PACKET)
-        log_printhex (array, 20, "keygrip=");
+        log_printhex ("keygrip=", array, 20);
       /* FIXME: Save the keygrip in PK.  */
     }
   gcry_sexp_release (s_pkey);
@@ -1190,42 +963,20 @@ gpg_error_t
 hexkeygrip_from_pk (PKT_public_key *pk, char **r_grip)
 {
   gpg_error_t err;
-  unsigned char grip[KEYGRIP_LEN];
+  unsigned char grip[20];
 
   *r_grip = NULL;
   err = keygrip_from_pk (pk, grip);
   if (!err)
     {
-      char * buf = xtrymalloc (KEYGRIP_LEN * 2 + 1);
+      char * buf = xtrymalloc (20*2+1);
       if (!buf)
         err = gpg_error_from_syserror ();
       else
         {
-          bin2hex (grip, KEYGRIP_LEN, buf);
+          bin2hex (grip, 20, buf);
           *r_grip = buf;
         }
     }
   return err;
 }
-
-
-/* Return a hexfied malloced string of the ECDH parameters for an ECDH
- * key from the public key PK.  Returns NULL on error.  */
-char *
-ecdh_param_str_from_pk (PKT_public_key *pk)
-{
-  const unsigned char *s;
-  unsigned int n;
-
-  if (!pk
-      || pk->pubkey_algo != PUBKEY_ALGO_ECDH
-      || !gcry_mpi_get_flag (pk->pkey[2], GCRYMPI_FLAG_OPAQUE)
-      || !(s = gcry_mpi_get_opaque (pk->pkey[2], &n)) || !n)
-    {
-      gpg_err_set_errno (EINVAL);
-      return NULL;  /* Invalid parameter */
-    }
-
-  n = (n+7)/8;
-  return bin2hex (s, n, NULL);
-}
