@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -26,13 +26,13 @@
 #include <errno.h>
 
 #include "gpgsm.h"
-#include "i18n.h"
+#include "../common/i18n.h"
 #include <ksba.h>
 
 
 /* We open the file only once and keep the open file pointer as well
    as the name of the file here.  Note that, a listname not equal to
-   NULL indicates that this module has been intialized and if the
+   NULL indicates that this module has been initialized and if the
    LISTFP is also NULL, no list of qualified signatures exists. */
 static char *listname;
 static FILE *listfp;
